//////////////////////////////////////////////////////////////////////////////
// Sketch: usb_ghost
// แสดงตัวอย่างการจำลองบอร์ดไมโครคอนโทรลเลอร์ให้เป็นอุปกรณ์คีย์บอร์ด เมาส์ และเกมแพด
// ในเวลาเดียวกัน
//
// == การคอมไพล์ ==
// ต้องการไลบรารี pt และ usbdrv (มีมาให้กับแพคเกจ cpeku-practicum เรียบร้อยแล้ว)
//
// == การใช้งาน ==
// ทำแสงบริเวณตัวรับแสงให้มืดเพื่อสั่งให้บอร์ดเริ่มส่งคีย์และขยับเมาส์
//
// == หมายเหตุ ==
// อุปกรณ์ USB ที่จำลองขึ้นกำหนดให้มีหมายเลข VID:PID เป็น 16c0:0482 ซึ่งได้รับการจดทะเบียน
// ให้ใช้งานเป็น Keyboard/Mouse/Joystick combm Practicumo โดย VOTI
//////////////////////////////////////////////////////////////////////////////
#include <pt.h>
#include <usbdrv.h>
// #define __usbdrv_h_included__

#include "keycodes.h"

#define REPORT_ID_KEYBOARD 1
#define REPORT_ID_MOUSE    2
#define REPORT_ID_GAMEPAD  3

#define LEFT_BUTTON PIN_PC0
#define DOWN_BUTTON PIN_PC1
#define RIGHT_BUTTON PIN_PC4
#define UP_BUTTON PIN_PC5

#define LEFT_BUTTON2  PIN_PD0
#define DOWN_BUTTON2 PIN_PD1
#define RIGHT_BUTTON2 PIN_PD4
#define UP_BUTTON2 PIN_PD5

// มาโครสำหรับจำลองการหน่วงเวลาใน protothread
#define PT_DELAY(pt,ms,tsVar) \
  tsVar = millis(); \
  PT_WAIT_UNTIL(pt, millis()-tsVar >= (ms));

/////////////////////////////////////////////////////////////////////
// USB report descriptor, สร้างขึ้นจาก HID Descriptor Tool ซึ่ง
// ดาวน์โหลดได้จาก
//    http://www.usb.org/developers/hidpage/dt2_4.zip
//
// หรือใช้ HIDEdit ซึ่งให้บริการฟรีผ่านเว็บที่ http://hidedit.org/
//
// *** ไม่แนะนำให้สร้างเองด้วยมือ ***
/////////////////////////////////////////////////////////////////////
PROGMEM const char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = 
{
  ///////////////////
  // Keyboard
  ///////////////////
  0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
  0x09, 0x06,                    // USAGE (Keyboard)
  0xa1, 0x01,                    // COLLECTION (Application)
  0x85, 0x01,                    //   REPORT_ID (1)
  0x05, 0x07,                    //   USAGE_PAGE (Keyboard)
  0x19, 0xe0,                    //   USAGE_MINIMUM (Keyboard LeftControl)
  0x29, 0xe7,                    //   USAGE_MAXIMUM (Keyboard Right GUI)
  0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
  0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
  0x75, 0x01,                    //   REPORT_SIZE (1)
  0x95, 0x08,                    //   REPORT_COUNT (8)
  0x81, 0x02,                    //   INPUT (Data,Var,Abs)
  0x95, 0x01,                    //   REPORT_COUNT (1)
  0x75, 0x08,                    //   REPORT_SIZE (8)
  0x25, 0x65,                    //   LOGICAL_MAXIMUM (101)
  0x19, 0x00,                    //   USAGE_MINIMUM (Reserved (no event indicated))
  0x29, 0x65,                    //   USAGE_MAXIMUM (Keyboard Application)
  0x81, 0x00,                    //   INPUT (Data,Ary,Abs)
  0xc0,                          // END_COLLECTION

  ///////////////////
  // 3-Button Mouse
  ///////////////////
  0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
  0x0b, 0x02, 0x00, 0x01, 0x00,  // USAGE (Generic Desktop:Mouse)
  0xa1, 0x01,                    // COLLECTION (Application)
  0x0b, 0x01, 0x00, 0x01, 0x00,  //   USAGE (Generic Desktop:Pointer)
  0xa1, 0x00,                    //   COLLECTION (Physical)
  0x85, 0x02,                    //     REPORT_ID (2)
  0x05, 0x09,                    //     USAGE_PAGE (Button)
  0x1b, 0x01, 0x00, 0x09, 0x00,  //     USAGE_MINIMUM (Button:Button 1)
  0x2b, 0x03, 0x00, 0x09, 0x00,  //     USAGE_MAXIMUM (Button:Button 3)
  0x15, 0x00,                    //     LOGICAL_MINIMUM (0)
  0x25, 0x01,                    //     LOGICAL_MAXIMUM (1)
  0x75, 0x01,                    //     REPORT_SIZE (1)
  0x95, 0x03,                    //     REPORT_COUNT (3)
  0x81, 0x02,                    //     INPUT (Data,Var,Abs)
  0x75, 0x05,                    //     REPORT_SIZE (5)
  0x95, 0x01,                    //     REPORT_COUNT (1)
  0x81, 0x03,                    //     INPUT (Cnst,Var,Abs)
  0x05, 0x01,                    //     USAGE_PAGE (Generic Desktop)
  0x0b, 0x30, 0x00, 0x01, 0x00,  //     USAGE (Generic Desktop:X)
  0x0b, 0x31, 0x00, 0x01, 0x00,  //     USAGE (Generic Desktop:Y)
  0x15, 0x81,                    //     LOGICAL_MINIMUM (-127)
  0x25, 0x7f,                    //     LOGICAL_MAXIMUM (127)
  0x75, 0x08,                    //     REPORT_SIZE (8)
  0x95, 0x02,                    //     REPORT_COUNT (2)
  0x81, 0x06,                    //     INPUT (Data,Var,Rel)
  0xc0,                          //     END_COLLECTION
  0xc0,                          // END_COLLECTION

  ////////////////////////////////////////////////////
  // Gamepad with one button and one analog stick
  ////////////////////////////////////////////////////
  0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
  0x09, 0x04,                    // USAGE (Joystick)
  0xa1, 0x01,                    // COLLECTION (Application)
  0x05, 0x01,                    //   USAGE_PAGE (Generic Desktop)
  0x09, 0x01,                    //   USAGE (Pointer)
  0xa1, 0x00,                    //   COLLECTION (Physical)
  0x85, 0x03,                    //     REPORT_ID (3)
  0x05, 0x01,                    //     USAGE_PAGE (Generic Desktop)
  0x09, 0x32,                    //     USAGE (Z)
  0x15, 0x00,                    //     LOGICAL_MINIMUM (0)
  0x26, 0xff, 0x03,              //     LOGICAL_MAXIMUM (1023)
  0x75, 0x0a,                    //     REPORT_SIZE (10)
  0x95, 0x01,                    //     REPORT_COUNT (1)
  0x81, 0x02,                    //     INPUT (Data,Var,Abs)
  0x05, 0x09,                    //     USAGE_PAGE (Button)
  0x09, 0x01,                    //     USAGE (Button 1)
  0x15, 0x00,                    //     LOGICAL_MINIMUM (0)
  0x25, 0x01,                    //     LOGICAL_MAXIMUM (1)
  0x75, 0x01,                    //     REPORT_SIZE (1)
  0x95, 0x01,                    //     REPORT_COUNT (1)
  0x81, 0x02,                    //     INPUT (Data,Var,Abs)
  0x75, 0x01,                    //     REPORT_SIZE (1)
  0x95, 0x05,                    //     REPORT_COUNT (5)
  0x81, 0x03,                    //     INPUT (Cnst,Var,Abs)
  0xc0,                          //   END_COLLECTION
  0xc0                           // END_COLLECTION
};

struct ReportKeyboard
{
  /* +----\------+-----+-----+-----+-----+-----+-----+-----+-----+
   * |Byte \ Bit |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |
   * +------\----+-----+-----+-----+-----+-----+-----+-----+-----+
   * |  0        |               Report ID = 1                   |
   * +-----------+-----+-----+-----+-----+-----+-----+-----+-----+
   * |  1        |           Modifiers (shift,ctrl,etc)          |
   * +-----------+-----+-----+-----+-----+-----+-----+-----+-----+
   * |  2        |                 Key Code                      |
   * +-----------+-----+-----+-----+-----+-----+-----+-----+-----+
   */
  uint8_t  report_id;
  uint8_t  modifiers;
  uint8_t  key_code;
};



ReportKeyboard reportKeyboard;


// Protothread states
struct pt main_pt;
struct pt keyboards_pt[4];
int j;

////////////////////////////////////////////////////////////////
// Automatically called by usbpoll() when host makes a request
////////////////////////////////////////////////////////////////
usbMsgLen_t usbFunctionSetup(uchar data[8])
{
  usbRequest_t *rq = (usbRequest_t*)data;

  return 0;  /* Return nothing to host for now */
}

//////////////////////////////////////////////////////////////
int sendKey(uint8_t keycode, uint8_t modifiers)
{
  reportKeyboard.key_code = keycode;
  reportKeyboard.modifiers = modifiers;
  usbSetInterrupt((uchar*)&reportKeyboard, sizeof(reportKeyboard));
  return 1;
}
  
int KEY_BUTTONS[] = { KEY_W, KEY_S, KEY_A, KEY_D };
//////////////////////////////////////////////////////////////
PT_THREAD(keyboard_task(struct pt *pt, uint8_t keycode, uint8_t pin))
{
  static uint32_t ts = 0;
  
  PT_BEGIN(pt);

  while (1)
  {
//    rightButton = !digitalRead(RIGHT_BUTTON);
     PT_WAIT_WHILE(pt, digitalRead(pin));
//    while(1){
//      if(!digitalRead(pin)) break;
//      PT_DELAY(pt, 50, ts);
//    }
    
    PT_WAIT_WHILE(pt, !digitalRead(pin) && (sendKey(keycode, 0)));
    // sendKey(KEY_NONE, 0);
    PT_YIELD(pt);
  }
  
  PT_END(pt);
}



//////////////////////////////////////////////////////////////
PT_THREAD(main_task(struct pt *pt))
{
  static int i;
  static uint32_t ts = 0;
  PT_BEGIN(pt);
  j = 0;
  for(i = 0; i < 4; i++){
    keyboard_task(&keyboards_pt[i], KEY_BUTTONS[i], BUTTONS[i]);
    PT_WAIT_UNTIL(pt,usbInterruptIsReady());
    //PT_DELAY(pt, 50, ts);
  }
  if(j == 0)
    sendKey(KEY_NONE, 0);
  PT_END(pt);
}

//////////////////////////////////////////////////////////////
void setup()
{
  // Initialize USB subsystem
  usbInit();
  usbDeviceDisconnect();
  delay(300);
  usbDeviceConnect();
  
  // Initialize hardware pins

  pinMode(PIN_PD3, OUTPUT);
  pinMode(RIGHT_BUTTON, INPUT_PULLUP);
  pinMode(LEFT_BUTTON, INPUT_PULLUP);
  pinMode(UP_BUTTON, INPUT_PULLUP);
  pinMode(DOWN_BUTTON, INPUT_PULLUP);

  // Initialize USB reports
  reportKeyboard.report_id = REPORT_ID_KEYBOARD;
  reportKeyboard.modifiers = 0;
  reportKeyboard.key_code = KEY_NONE;


  // Initialize tasks
  PT_INIT(&main_pt);
  for(int i = 0; i < 4; i++)
    PT_INIT(&keyboards_pt[i]);
}

//////////////////////////////////////////////////////////////
void loop()
{
  usbPoll();
  main_task(&main_pt);
}
