# - * - coding: utf-8 - * -

import kivy
kivy.require('1.9.0')
from kivy.uix.widget import Widget
from kivy.app import App
from kivy.uix.floatlayout import FloatLayout
from kivy.uix.gridlayout import GridLayout
from kivy.uix.image import Image
from kivy.lang import Builder
from kivy.animation import Animation, AnimationTransition
from kivy.properties import NumericProperty, ReferenceListProperty,\
    ObjectProperty, ListProperty, BooleanProperty
from kivy.clock import Clock
from kivy.vector import Vector
from kivy.core.window import Window
from random import randint
from kivy.core.audio import SoundLoader
from kivy.uix.screenmanager import ScreenManager, Screen, FadeTransition
from kivy.vector import Vector
import sys
from kivy.config import Config

JUMP_VELOCITY = 25.0
GRAVITY_ACCELERATION = -1.5
MOVE_ACCELERATION = 1.0
MAX_VELOCITY_X = 15.0

class Player(Widget):
    landed = NumericProperty(0)
    velocity_y = NumericProperty(0.0)
    velocity_x = NumericProperty(0.0)
    acceleration_y = NumericProperty(GRAVITY_ACCELERATION)
    is_jumped = BooleanProperty(False)
    mass = NumericProperty(50.0)
    a = NumericProperty(255)
    r = NumericProperty(255)
    g = NumericProperty(255)
    b = NumericProperty(255)

    @property
    def low_y(self):
        return self.y - self.height

    @low_y.setter
    def low_y(self, value):
        self.y = value + self.height

    def update(self):
        self.update_y()
        self.update_x()
        player_collision = self.is_player_collision()
        #try:
        #    if player_collision:
        #        self.velocity_x = 2.0 * Vector(player_collision.pos) [0]
        #        player_collision.velocity_x =  2.0 * Vector(self.pos) [0]
        #except ZeroDivisionError:
        #    pass

    def update_y(self):
        self.gravity()
        self.y += self.velocity_y
        if self.low_y < self.landed:
            self.low_y = self.landed
            self.velocity_y = 0.0
            self.is_jumped= False
        elif self.y < 0:
            self.y = 0
        else:
            pass
       
    def gravity(self):
        self.velocity_y += self.acceleration_y

    def jump(self):
        if not self.is_jumped:
            self.is_jumped= True
            self.velocity_y += JUMP_VELOCITY

    def move(self, reverse=False):
        vel = MOVE_ACCELERATION
        if reverse:
            vel *= -1
        self.velocity_x += vel

    def update_x(self):
        if self.velocity_x != 0.0:
            self.velocity_x += -0.5 * (self.velocity_x / abs(self.velocity_x))
            if abs(self.velocity_x) > MAX_VELOCITY_X:
                self.velocity_x = MAX_VELOCITY_X * (self.velocity_x / abs(self.velocity_x))
            self.x -= self.velocity_x
        if self.x > self.parent.width - self.width:
            self.velocity_x *= -1
            self.x = self.parent.width - self.width
        elif self.x < 0:
            self.x = 0
            self.velocity_x *= -1
    
    def is_player_collision(self):
        for player in self.parent.players:
            if player != self:
                if self.collide_widget(player):
                    if self.x + (self.width * 0.95) > player.x and player.x > self.x:
                        self.velocity_x = -5.0
                        player.velocity_x = 5.0

                    newplayer_vel_x = (self.velocity_x * self.mass) / (player.mass)
                    newplayer_vel_y = (self.velocity_y * self.mass) / (player.mass)
                    newself_vel_x = (player.velocity_x * player.mass) / (self.mass)
                    newself_vel_y = (player.velocity_y * player.mass) / (self.mass)
                    # self.velocity_y = newself_vel_y
                    self.velocity_x = newself_vel_x
                    # player.velocity_y = newplayer_vel_y
                    player.velocity_x = newplayer_vel_x
                    return player
        return None


